<?php

/**
 * Record Actions test class.
 *
 * @copyright YetiForce Sp. z o.o
 * @license   YetiForce Public License 3.0 (licenses/LicenseEN.txt or yetiforce.com)
 * @author    Mariusz Krzaczkowski <m.krzaczkowski@yetiforce.com>
 */

namespace Tests\Base;

class C_RecordActions extends \Tests\Base
{
	/**
	 * Temporary record object.
	 *
	 * @var \Vtiger_Record_Model
	 */
	protected static $recordAccounts;
	/**
	 * Temporary record object.
	 *
	 * @var \Vtiger_Record_Model
	 */
	protected static $recordCampaigns;
	/**
	 * Temporary Contacts record object.
	 *
	 * @var \Vtiger_Record_Model
	 */
	protected static $recordContacts;
	/**
	 * Temporary Products record object.
	 *
	 * @var \Vtiger_Record_Model
	 */
	protected static $recordProducts;
	/**
	 * Temporary SQuotes record object.
	 *
	 * @var \Vtiger_Record_Model
	 */
	protected static $recordSQuotes;
	/**
	 * Temporary Leads record object.
	 *
	 * @var \Vtiger_Record_Model
	 */
	protected static $recordLeads;
	/**
	 * Temporary loremIpsum text.
	 *
	 * @var string
	 */
	protected static $loremIpsumText;
	/**
	 * Temporary loremIpsum html.
	 *
	 * @var string
	 */
	protected static $loremIpsumHtml;

	/**
	 * Load lorem ipsum clear text for tests.
	 *
	 * @return string
	 */
	public static function createLoremIpsumText()
	{
		if (static::$loremIpsumText) {
			return static::$loremIpsumText;
		}
		return static::$loremIpsumText = \file_get_contents('./tests/data/stringText.txt');
	}

	/**
	 * Load lorem ipsum html text for tests.
	 *
	 * @return string
	 */
	public static function createLoremIpsumHtml()
	{
		if (static::$loremIpsumHtml) {
			return static::$loremIpsumHtml;
		}
		return static::$loremIpsumHtml = \file_get_contents('./tests/data/stringHtml.txt');
	}

	/**
	 * Creating leads module record for tests.
	 *
	 * @var bool
	 *
	 * @param mixed $cache
	 *
	 * @return \Vtiger_Record_Model
	 */
	public static function createLeadRecord($cache = true)
	{
		if (static::$recordLeads && $cache) {
			return static::$recordLeads;
		}
		$recordModel = \Vtiger_Record_Model::getCleanInstance('Leads');
		$recordModel->set('company', 'TestLead sp. z o.o.');
		$recordModel->set('description', 'autogenerated test lead for \App\TextParser tests');
		$recordModel->set('assigned_user_id', \App\User::getCurrentUserId());
		$recordModel->save();
		return static::$recordLeads = $recordModel;
	}

	/**
	 * Creating contacts module record for tests.
	 *
	 * @param mixed $cache
	 *
	 * @return \Vtiger_Record_Model
	 */
	public static function createContactRecord($cache = true)
	{
		if (static::$recordContacts && $cache) {
			return static::$recordContacts;
		}
		$recordModel = \Vtiger_Record_Model::getCleanInstance('Contacts');
		$recordModel->set('salutationtype', 'Mr.');
		$recordModel->set('firstname', 'Test');
		$recordModel->set('lastname', 'Testowy');
		$recordModel->set('contactstatus', 'Active');
		$recordModel->set('verification', 'Address details');
		$recordModel->set('parent_id', static::createAccountRecord()->getId());
		$recordModel->set('assigned_user_id', \App\User::getCurrentUserId());
		$recordModel->save();
		return static::$recordContacts = $recordModel;
	}

	/**
	 * Creating account module record for tests.
	 *
	 * @var bool
	 *
	 * @param mixed $cache
	 */
	public static function createAccountRecord($cache = true)
	{
		if (static::$recordAccounts && $cache) {
			return static::$recordAccounts;
		}
		$record = \Vtiger_Record_Model::getCleanInstance('Accounts');
		$record->set('accountname', 'YetiForce Sp. z o.o.');
		$record->set('legal_form', 'PLL_GENERAL_PARTNERSHIP');
		$record->save();
		static::$recordAccounts = $record;
		return $record;
	}

	/**
	 * Creating Product module record for tests.
	 *
	 * @var bool
	 *
	 * @param mixed $cache
	 */
	public static function createSQuotesRecord($cache = true)
	{
		if (static::$recordSQuotes && $cache) {
			return static::$recordSQuotes;
		}
		$record = \Vtiger_Record_Model::getCleanInstance('SQuotes');
		$record->set('subject', 'System CRM YetiForce');
		$record->setInventoryItemPart(1, 'name', self::createProductRecord()->getId());
		$record->setInventoryItemPart(1, 'discountmode', 1);
		$record->setInventoryItemPart(1, 'taxmode', 1);
		$record->setInventoryItemPart(1, 'currency', 1);
		$record->setInventoryItemPart(1, 'qty', 2);
		$record->setInventoryItemPart(1, 'price', 5);
		$record->setInventoryItemPart(1, 'total', 10);
		$record->setInventoryItemPart(1, 'discount', 0);
		$record->setInventoryItemPart(1, 'net', 10);
		$record->setInventoryItemPart(1, 'purchase', 0);
		$record->setInventoryItemPart(1, 'marginp', 0);
		$record->setInventoryItemPart(1, 'margin', 0);
		$record->setInventoryItemPart(1, 'tax', 0);
		$record->setInventoryItemPart(1, 'gross', 10);
		$record->setInventoryItemPart(1, 'comment1', '');
		$record->save();
		static::$recordSQuotes = $record;
		return $record;
	}

	/**
	 * Creating Product module record for tests.
	 *
	 * @var bool
	 *
	 * @param mixed $cache
	 */
	public static function createProductRecord($cache = true)
	{
		if (static::$recordProducts && $cache) {
			return static::$recordProducts;
		}
		$record = \Vtiger_Record_Model::getCleanInstance('Products');
		$record->set('productname', 'System CRM YetiForce');
		$record->save();
		static::$recordProducts = $record;
		return $record;
	}

	/**
	 * Testing the record creation.
	 */
	public function testCreateRecord()
	{
		static::assertIsInt(static::createAccountRecord()->getId());
	}

	/**
	 * Testing editing permissions.
	 */
	public function testPermission()
	{
		$this->assertTrue(static::$recordAccounts->isEditable());
		$this->assertTrue(static::$recordAccounts->isCreateable());
		$this->assertTrue(static::$recordAccounts->isViewable());
		$this->assertFalse(static::$recordAccounts->privilegeToActivate());
		$this->assertTrue(static::$recordAccounts->privilegeToArchive());
		$this->assertTrue(static::$recordAccounts->privilegeToMoveToTrash());
		$this->assertTrue(static::$recordAccounts->privilegeToDelete());
	}

	/**
	 * Testing the edit block feature.
	 */
	public function testCheckLockFields()
	{
		$this->assertFalse(static::$recordAccounts->isLockByFields());
	}

	/**
	 * Testing record editing.
	 */
	public function testEditRecord()
	{
		static::$recordAccounts->set('accounttype', 'Customer');
		static::$recordAccounts->save();
		$this->assertTrue((new \App\Db\Query())->from('vtiger_account')->where(['account_type' => 'Customer'])->exists());
	}

	/**
	 * Testing the record label.
	 */
	public function testGetDisplayName()
	{
		$this->assertTrue('YetiForce Sp. z o.o.' === static::$recordAccounts->getDisplayName());
	}

	/**
	 * Testing the change record state.
	 */
	public function testStateRecord()
	{
		static::$recordAccounts->changeState('Trash');
		$this->assertSame(1, (new \App\Db\Query())->select(['deleted'])->from('vtiger_crmentity')->where(['crmid' => static::$recordAccounts->getId()])->scalar());
		static::$recordAccounts->changeState('Active');
		$this->assertSame(0, (new \App\Db\Query())->select(['deleted'])->from('vtiger_crmentity')->where(['crmid' => static::$recordAccounts->getId()])->scalar());
		static::$recordAccounts->changeState('Archived');
		$this->assertSame(2, (new \App\Db\Query())->select(['deleted'])->from('vtiger_crmentity')->where(['crmid' => static::$recordAccounts->getId()])->scalar());
	}
}
